/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMElement;
interface nsIController;
interface nsIControllers;
interface nsIDOMWindow;


[scriptable, uuid(f3c50361-14fe-11d3-bf87-00105a1b0627)]
interface nsIDOMXULCommandDispatcher : nsISupports
{
            attribute nsIDOMElement    focusedElement;
            attribute nsIDOMWindow     focusedWindow;

  void                      addCommandUpdater(in nsIDOMElement updater,
                                              in DOMString events,
                                              in DOMString targets);
  void                      removeCommandUpdater(in nsIDOMElement updater);

  void                      updateCommands(in DOMString eventName);

  nsIController             getControllerForCommand(in string command);
  nsIControllers            getControllers();

  void advanceFocus();
  void rewindFocus();
  void advanceFocusIntoSubtree(in nsIDOMElement elt);
  attribute boolean suppressFocusScroll;
};
