/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMDOMRequest;
interface nsIDOMWindow;

/**
 * XPCOM component (in the content process) that provides the ICC information.
 */
[scriptable, uuid(09f29ef0-006f-11e5-a0e1-fbef06958398)]
interface nsIIccProvider : nsISupports
{
  /**
   * Phonebook interfaces.
   */
  nsIDOMDOMRequest readContacts(in unsigned long clientId,
                                in nsIDOMWindow window,
                                in unsigned long contactType);

  nsIDOMDOMRequest updateContact(in unsigned long clientId,
                                 in nsIDOMWindow window,
                                 in unsigned long contactType,
                                 in jsval contact,
                                 in DOMString pin2);
};
